@@ -29,7 +29,7 @@ Do reads alignment with STAR.
 =cut
 
 package CLIPSeqTools::PreprocessApp::star_alignment;
-$CLIPSeqTools::PreprocessApp::star_alignment::VERSION = '0.1.1';
+$CLIPSeqTools::PreprocessApp::star_alignment::VERSION = '0.1.2';
 
 # Make it an app command
 use MooseX::App::Command;
@@ -116,6 +116,10 @@ sub run {
 		'--outFileNamePrefix ' . $self->o_prefix. 'star_',
 	);
 
+	if ($self->fastq =~ /\.gz$/) {
+		$cmd .= ' ' . '--readFilesCommand zcat';
+	}
+
 	warn "Creating output path\n" if $self->verbose;
 	$self->make_path_for_output_prefix();
 
